@@ -1,6 +1,6 @@
 /* metaflac - Command-line FLAC metadata editor
  * Copyright (C) 2001-2009  Josh Coalson
- * Copyright (C) 2011-2014  Xiph.Org Foundation
+ * Copyright (C) 2011-2016  Xiph.Org Foundation
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -79,6 +79,7 @@ struct share__option long_options_[] = {
 	{ "export-picture-to", 1, 0, 0 },
 	{ "add-seekpoint", 1, 0, 0 },
 	{ "add-replay-gain", 0, 0, 0 },
+	{ "scan-replay-gain", 0, 0, 0 },
 	{ "remove-replay-gain", 0, 0, 0 },
 	{ "add-padding", 1, 0, 0 },
 	/* major operations */
@@ -612,6 +613,9 @@ FLAC__bool parse_option(int option_index, const char *option_argument, CommandLi
 	else if(0 == strcmp(opt, "add-replay-gain")) {
 		(void) append_shorthand_operation(options, OP__ADD_REPLAY_GAIN);
 	}
+	else if(0 == strcmp(opt, "scan-replay-gain")) {
+		(void) append_shorthand_operation(options, OP__SCAN_REPLAY_GAIN);
+	}
 	else if(0 == strcmp(opt, "remove-replay-gain")) {
 		const FLAC__byte * const tags[5] = {
 			GRABBAG__REPLAYGAIN_TAG_REFERENCE_LOUDNESS,
@@ -1023,7 +1027,7 @@ FLAC__bool parse_block_type(const char *in, Argument_BlockType *out)
 				if(strlen(r) == sizeof (out->entries[entry].application_id)) {
 					memcpy(out->entries[entry].application_id, r, sizeof (out->entries[entry].application_id));
 				}
-				else if(strlen(r) == 10 && strncmp(r, "0x", 2) == 0 && strspn(r+2, "0123456789ABCDEFabcdef") == 8) {
+				else if(strlen(r) == 10 && FLAC__STRNCASECMP(r, "0x", 2) == 0 && strspn(r+2, "0123456789ABCDEFabcdef") == 8) {
 					FLAC__uint32 x = strtoul(r+2, 0, 16);
 					out->entries[entry].application_id[3] = (FLAC__byte)(x & 0xff);
 					out->entries[entry].application_id[2] = (FLAC__byte)((x>>=8) & 0xff);
