@@ -18,6 +18,10 @@
  *  with this program; if not, write to the Free Software Foundation, Inc.,
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "plugin.h"
 
 #include <stdlib.h>
