@@ -1,6 +1,6 @@
 /* test_streams - Simple test pattern generator
  * Copyright (C) 2000-2009  Josh Coalson
- * Copyright (C) 2011-2014  Xiph.Org Foundation
+ * Copyright (C) 2011-2016  Xiph.Org Foundation
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,7 +43,7 @@
 static FLAC__bool is_big_endian_host;
 
 
-static FLAC__bool write_little_endian(FILE *f, FLAC__int32 x, size_t bytes)
+static FLAC__bool write_little_endian_unsigned(FILE *f, FLAC__uint32 x, size_t bytes)
 {
 	while(bytes) {
 		if(fputc(x, f) == EOF)
@@ -54,6 +54,11 @@ static FLAC__bool write_little_endian(FILE *f, FLAC__int32 x, size_t bytes)
 	return true;
 }
 
+static FLAC__bool write_little_endian_signed(FILE *f, FLAC__int32 x, size_t bytes)
+{
+	return write_little_endian_unsigned(f, (FLAC__uint32) x, bytes);
+}
+
 static FLAC__bool write_little_endian_uint16(FILE *f, FLAC__uint16 x)
 {
 	return
@@ -217,7 +222,7 @@ static FLAC__bool generate_01(void)
 	FILE *f;
 	FLAC__int16 x = -32768;
 
-	if(0 == (f = flac_fopen("test01.raw", "wb")))
+	if(0 == (f = fopen("test01.raw", "wb")))
 		return false;
 
 	if(!write_little_endian_int16(f, x))
@@ -236,7 +241,7 @@ static FLAC__bool generate_02(void)
 	FILE *f;
 	FLAC__int16 xl = -32768, xr = 32767;
 
-	if(0 == (f = flac_fopen("test02.raw", "wb")))
+	if(0 == (f = fopen("test02.raw", "wb")))
 		return false;
 
 	if(!write_little_endian_int16(f, xl))
@@ -258,7 +263,7 @@ static FLAC__bool generate_03(void)
 	FLAC__int16 x[] = { -25, 0, 25, 50, 100 };
 	unsigned i;
 
-	if(0 == (f = flac_fopen("test03.raw", "wb")))
+	if(0 == (f = fopen("test03.raw", "wb")))
 		return false;
 
 	for(i = 0; i < 5; i++)
@@ -279,7 +284,7 @@ static FLAC__bool generate_04(void)
 	FLAC__int16 x[] = { -25, 500, 0, 400, 25, 300, 50, 200, 100, 100 };
 	unsigned i;
 
-	if(0 == (f = flac_fopen("test04.raw", "wb")))
+	if(0 == (f = fopen("test04.raw", "wb")))
 		return false;
 
 	for(i = 0; i < 10; i++)
@@ -301,7 +306,7 @@ static FLAC__bool generate_fsd8(const char *fn, const int pattern[], unsigned re
 
 	FLAC__ASSERT(pattern != 0);
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(rep = 0; rep < reps; rep++) {
@@ -327,7 +332,7 @@ static FLAC__bool generate_fsd16(const char *fn, const int pattern[], unsigned r
 
 	FLAC__ASSERT(pattern != 0);
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(rep = 0; rep < reps; rep++) {
@@ -351,7 +356,7 @@ static FLAC__bool generate_wbps16(const char *fn, unsigned samples)
 	FILE *f;
 	unsigned sample;
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(sample = 0; sample < samples; sample++) {
@@ -378,7 +383,7 @@ static FLAC__bool generate_fsd24(const char *fn, const int pattern[], unsigned r
 
 	FLAC__ASSERT(pattern != 0);
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(rep = 0; rep < reps; rep++) {
@@ -406,7 +411,7 @@ static FLAC__bool generate_sine8_1(const char *fn, const double sample_rate, con
 	double theta1, theta2;
 	unsigned i;
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(i = 0, theta1 = theta2 = 0.0; i < samples; i++, theta1 += delta1, theta2 += delta2) {
@@ -433,7 +438,7 @@ static FLAC__bool generate_sine8_2(const char *fn, const double sample_rate, con
 	double theta1, theta2;
 	unsigned i;
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(i = 0, theta1 = theta2 = 0.0; i < samples; i++, theta1 += delta1, theta2 += delta2) {
@@ -464,7 +469,7 @@ static FLAC__bool generate_sine16_1(const char *fn, const double sample_rate, co
 	double theta1, theta2;
 	unsigned i;
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(i = 0, theta1 = theta2 = 0.0; i < samples; i++, theta1 += delta1, theta2 += delta2) {
@@ -491,7 +496,7 @@ static FLAC__bool generate_sine16_2(const char *fn, const double sample_rate, co
 	double theta1, theta2;
 	unsigned i;
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(i = 0, theta1 = theta2 = 0.0; i < samples; i++, theta1 += delta1, theta2 += delta2) {
@@ -522,7 +527,7 @@ static FLAC__bool generate_sine24_1(const char *fn, const double sample_rate, co
 	double theta1, theta2;
 	unsigned i;
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(i = 0, theta1 = theta2 = 0.0; i < samples; i++, theta1 += delta1, theta2 += delta2) {
@@ -549,7 +554,7 @@ static FLAC__bool generate_sine24_2(const char *fn, const double sample_rate, co
 	double theta1, theta2;
 	unsigned i;
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(i = 0, theta1 = theta2 = 0.0; i < samples; i++, theta1 += delta1, theta2 += delta2) {
@@ -575,7 +580,7 @@ static FLAC__bool generate_noise(const char *fn, unsigned bytes)
 	FILE *f;
 	unsigned b;
 
-	if(0 == (f = flac_fopen(fn, "wb")))
+	if(0 == (f = fopen(fn, "wb")))
 		return false;
 
 	for(b = 0; b < bytes; b++) {
@@ -595,7 +600,7 @@ foo:
 	return false;
 }
 
-static FLAC__bool generate_raw(const char *filename, unsigned channels, unsigned bytes_per_sample, unsigned samples)
+static FLAC__bool generate_signed_raw(const char *filename, unsigned channels, unsigned bytes_per_sample, unsigned samples)
 {
 	const FLAC__int32 full_scale = (1 << (bytes_per_sample*8-1)) - 1;
 	const double f1 = 441.0, a1 = 0.61, f2 = 661.5, a2 = 0.37;
@@ -605,14 +610,44 @@ static FLAC__bool generate_raw(const char *filename, unsigned channels, unsigned
 	FILE *f;
 	unsigned i, j;
 
-	if(0 == (f = flac_fopen(filename, "wb")))
+	if(0 == (f = fopen(filename, "wb")))
 		return false;
 
 	for(i = 0, theta1 = theta2 = 0.0; i < samples; i++, theta1 += delta1, theta2 += delta2) {
 		for(j = 0; j < channels; j++) {
 			double val = (a1*sin(theta1) + a2*sin(theta2))*(double)full_scale;
 			FLAC__int32 v = (FLAC__int32)(val + 0.5) + ((GET_RANDOM_BYTE>>4)-8);
-			if(!write_little_endian(f, v, bytes_per_sample))
+			if(!write_little_endian_signed(f, v, bytes_per_sample))
+				goto foo;
+		}
+	}
+
+	fclose(f);
+	return true;
+foo:
+	fclose(f);
+	return false;
+}
+
+static FLAC__bool generate_unsigned_raw(const char *filename, unsigned channels, unsigned bytes_per_sample, unsigned samples)
+{
+	const FLAC__int32 full_scale = (1 << (bytes_per_sample*8-1)) - 1;
+	const double f1 = 441.0, a1 = 0.61, f2 = 661.5, a2 = 0.37;
+	const double delta1 = 2.0 * M_PI / ( 44100.0 / f1);
+	const double delta2 = 2.0 * M_PI / ( 44100.0 / f2);
+	const double half_scale = 0.5 * full_scale;
+	double theta1, theta2;
+	FILE *f;
+	unsigned i, j;
+
+	if(0 == (f = fopen(filename, "wb")))
+		return false;
+
+	for(i = 0, theta1 = theta2 = 0.0; i < samples; i++, theta1 += delta1, theta2 += delta2) {
+		for(j = 0; j < channels; j++) {
+			double val = (a1*sin(theta1) + a2*sin(theta2))*(double)full_scale;
+			FLAC__int32 v = (FLAC__int32)(half_scale + val + 0.5) + ((GET_RANDOM_BYTE>>4)-8);
+			if(!write_little_endian_unsigned(f, v, bytes_per_sample))
 				goto foo;
 		}
 	}
@@ -638,7 +673,7 @@ static FLAC__bool generate_aiff(const char *filename, unsigned sample_rate, unsi
 	FILE *f;
 	unsigned i, j;
 
-	if(0 == (f = flac_fopen(filename, "wb")))
+	if(0 == (f = fopen(filename, "wb")))
 		return false;
 	if(fwrite("FORM", 1, 4, f) < 4)
 		goto foo;
@@ -702,7 +737,7 @@ static FLAC__bool generate_wav(const char *filename, unsigned sample_rate, unsig
 	FILE *f;
 	unsigned i, j;
 
-	if(0 == (f = flac_fopen(filename, "wb")))
+	if(0 == (f = fopen(filename, "wb")))
 		return false;
 	/* RIFFxxxxWAVE or equivalent: */
 	switch(flavor) {
@@ -815,7 +850,7 @@ static FLAC__bool generate_wav(const char *filename, unsigned sample_rate, unsig
 		for(j = 0; j < channels; j++) {
 			double val = (a1*sin(theta1) + a2*sin(theta2))*(double)full_scale;
 			FLAC__int32 v = ((FLAC__int32)(val + 0.5) + ((GET_RANDOM_BYTE>>4)-8)) << shift;
-			if(!write_little_endian(f, v, bytes_per_sample))
+			if(!write_little_endian_signed(f, v, bytes_per_sample))
 				goto foo;
 		}
 	}
@@ -848,14 +883,14 @@ static FLAC__bool generate_wackywavs(void)
 		  4,   0,   0,   0, 'b', 'l', 'a', 'h'
 	};
 
-	if(0 == (f = flac_fopen("wacky1.wav", "wb")))
+	if(0 == (f = fopen("wacky1.wav", "wb")))
 		return false;
 	if(fwrite(wav, 1, 84, f) < 84)
 		goto foo;
 	fclose(f);
 
 	wav[4] += 12;
-	if(0 == (f = flac_fopen("wacky2.wav", "wb")))
+	if(0 == (f = fopen("wacky2.wav", "wb")))
 		return false;
 	if(fwrite(wav, 1, 96, f) < 96)
 		goto foo;
@@ -867,24 +902,62 @@ foo:
 	return false;
 }
 
+static FLAC__bool write_simple_wavex_header (FILE * f, unsigned samplerate, unsigned channels, unsigned bytespersample, unsigned frames)
+{
+	unsigned datalen = channels * bytespersample * frames ;
+
+	if (fwrite("RIFF", 1, 4, f) != 4)
+		return false;
+	if (!write_little_endian_uint32(f, 40 + 4 + 4 + datalen))
+		return false;
+
+	if (fwrite("WAVEfmt ", 8, 1, f) != 1)
+		return false;
+	if (!write_little_endian_uint32(f, 40))
+		return false;
+
+	if(!write_little_endian_uint16(f, 65534)) /* WAVEFORMATEXTENSIBLE tag */
+		return false;
+	if(!write_little_endian_uint16(f, channels))
+		return false;
+	if(!write_little_endian_uint32(f, samplerate))
+		return false;
+	if(!write_little_endian_uint32(f, samplerate * channels * bytespersample))
+		return false;
+	if(!write_little_endian_uint16(f, channels * bytespersample)) /* block align */
+		return false;
+	if(!write_little_endian_uint16(f, bytespersample * 8))
+		return false;
+
+	if(!write_little_endian_uint16(f, 22)) /* cbSize */
+		return false;
+	if(!write_little_endian_uint16(f, bytespersample * 8)) /* validBitsPerSample */
+		return false;
+	if(!write_little_endian_uint32(f, 0)) /* channelMask */
+		return false;
+	/* GUID = {0x00000001, 0x0000, 0x0010, {0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}} */
+	if(fwrite("\x01\x00\x00\x00\x00\x00\x10\x00\x80\x00\x00\xaa\x00\x38\x9b\x71", 1, 16, f) != 16)
+		return false;
+
+	if (fwrite("data", 1, 4, f) != 4)
+		return false;
+	if (!write_little_endian_uint32(f, datalen))
+		return false;
+
+	return true;
+}
+
 static FLAC__bool generate_noisy_sine(void)
 {
 	FILE *f;
-	FLAC__byte wav[] = {
-		'R', 'I', 'F', 'F',  76,   0,   0,   0,
-		'W', 'A', 'V', 'E', 'f', 'm', 't', ' ',
-		 16,   0,   0,   0,   1,   0,   1,   0,
-		0x44,0xAC,  0,   0,0x88,0x58,0x01,   0,
-		  2,   0,  16,   0, 'd', 'a', 't', 'a',
-		0xa8,   0xba,   0x6,   0
-	};
 	int32_t randstate = 0x1243456;
 	double sample, last_val = 0.0;
 	int k;
 
-	if(0 == (f = flac_fopen("noisy-sine.wav", "wb")))
+	if(0 == (f = fopen("noisy-sine.wav", "wb")))
 		return false;
-	if(fwrite(wav, 1, sizeof (wav), f) < sizeof (wav))
+
+	if(!write_simple_wavex_header (f, 44100, 1, 2, 220500))
 		goto foo;
 
 	for (k = 0 ; k < 5 * 44100 ; k++) {
@@ -944,14 +1017,14 @@ static FLAC__bool generate_wackywav64s(void)
 		 'b', 'l', 'a', 'h', 'b', 'l', 'a', 'h'
 	};
 
-	if(0 == (f = flac_fopen("wacky1.w64", "wb")))
+	if(0 == (f = fopen("wacky1.w64", "wb")))
 		return false;
 	if(fwrite(wav, 1, wav[16], f) < wav[16])
 		goto foo;
 	fclose(f);
 
 	wav[16] += 32;
-	if(0 == (f = flac_fopen("wacky2.w64", "wb")))
+	if(0 == (f = fopen("wacky2.w64", "wb")))
 		return false;
 	if(fwrite(wav, 1, wav[16], f) < wav[16])
 		goto foo;
@@ -986,14 +1059,14 @@ static FLAC__bool generate_wackyrf64s(void)
 		  4,   0,   0,   0, 'b', 'l', 'a', 'h'
 	};
 
-	if(0 == (f = flac_fopen("wacky1.rf64", "wb")))
+	if(0 == (f = fopen("wacky1.rf64", "wb")))
 		return false;
 	if(fwrite(wav, 1, 120, f) < 120)
 		goto foo;
 	fclose(f);
 
 	wav[20] += 12;
-	if(0 == (f = flac_fopen("wacky2.rf64", "wb")))
+	if(0 == (f = fopen("wacky2.rf64", "wb")))
 		return false;
 	if(fwrite(wav, 1, 132, f) < 132)
 		goto foo;
@@ -1005,6 +1078,40 @@ foo:
 	return false;
 }
 
+static FLAC__bool generate_replaygain_tone (unsigned samplerate)
+{
+	FILE *f;
+	char fname [256] ;
+	double tone, sample, samplerange;
+	int k;
+
+	flac_snprintf(fname, sizeof(fname), "rpg-tone-%u.wav", samplerate);
+
+	if(0 == (f = fopen(fname, "wb")))
+		return false;
+
+	if(!write_simple_wavex_header (f, samplerate, 1, 3, 220500))
+		goto foo;
+
+
+	samplerange = 0x7fffff; /* Largest sample value allowed for a 24 bit PCM file. */
+	tone = 1000.0; /* 1 kHz */
+
+	for (k = 0 ; k < 5 * 44100 ; k++) {
+		sample = sin(2 * M_PI * tone * k / samplerate);
+		sample *= samplerange;
+		if (!write_little_endian_uint24(f, (FLAC__int32) sample))
+			goto foo;
+	};
+
+	fclose(f);
+
+	return true;
+foo:
+	fclose(f);
+	return false;
+}
+
 int main(int argc, char *argv[])
 {
 	FLAC__uint32 test = 1;
@@ -1118,6 +1225,22 @@ int main(int argc, char *argv[])
 	if(!generate_sine24_2("sine24-18.raw", 44100.0, 200000, 441.0, 0.50, 4410.0, 0.49, 1.3)) return 1;
 	if(!generate_sine24_2("sine24-19.raw", 44100.0, 200000, 8820.0, 0.70, 4410.0, 0.29, 0.1)) return 1;
 
+	if(!generate_replaygain_tone(8000)) return 1;
+	if(!generate_replaygain_tone(11025)) return 1;
+	if(!generate_replaygain_tone(12000)) return 1;
+	if(!generate_replaygain_tone(16000)) return 1;
+	if(!generate_replaygain_tone(18900)) return 1;
+	if(!generate_replaygain_tone(22050)) return 1;
+	if(!generate_replaygain_tone(24000)) return 1;
+	if(!generate_replaygain_tone(28000)) return 1;
+	if(!generate_replaygain_tone(32000)) return 1;
+	if(!generate_replaygain_tone(36000)) return 1;
+	if(!generate_replaygain_tone(37800)) return 1;
+	if(!generate_replaygain_tone(44100)) return 1;
+	if(!generate_replaygain_tone(48000)) return 1;
+	if(!generate_replaygain_tone(96000)) return 1;
+	if(!generate_replaygain_tone(192000)) return 1;
+
 	/* WATCHOUT: the size of noise.raw is hardcoded into test/test_flac.sh */
 	if(!generate_noise("noise.raw", 65536 * 8 * 3)) return 1;
 	if(!generate_noise("noise8m32.raw", 32)) return 1;
@@ -1150,8 +1273,11 @@ int main(int argc, char *argv[])
 					return 1;
 
 				if(bits_per_sample % 8 == 0) {
-					flac_snprintf(fn, sizeof (fn), "rt-%u-%u-%u.raw", channels, bits_per_sample, nsamples[samples]);
-					if(!generate_raw(fn, channels, bits_per_sample/8, nsamples[samples]))
+					flac_snprintf(fn, sizeof (fn), "rt-%u-%u-signed-%u.raw", channels, bits_per_sample, nsamples[samples]);
+					if(!generate_signed_raw(fn, channels, bits_per_sample/8, nsamples[samples]))
+						return 1;
+					flac_snprintf(fn, sizeof (fn), "rt-%u-%u-unsigned-%u.raw", channels, bits_per_sample, nsamples[samples]);
+					if(!generate_unsigned_raw(fn, channels, bits_per_sample/8, nsamples[samples]))
 						return 1;
 				}
 			}
