@@ -1,6 +1,6 @@
 /* flactimer - Runs a command and prints timing information
  * Copyright (C) 2007-2009  Josh Coalson
- * Copyright (C) 2011-2014  Xiph.Org Foundation
+ * Copyright (C) 2011-2016  Xiph.Org Foundation
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -81,7 +81,7 @@ int main(int argc, char *argv[])
 		}
 		fout = fopen(argv[1], "w");
 		if(!fout) {
-			fprintf(fout, "ERROR opening file %s for writing\n", argv[1]);
+			fprintf(stderr, "ERROR opening file %s for writing\n", argv[1]);
 			return 1;
 		}
 		argv += 2;
